#
# spec file for package libnettle
#
# Copyright (c) 2019 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%define soname 6
%define hogweed_soname 4
Name:           libnettle
Version:        3.4.1
Release:        lp152.4.3.1
Summary:        Cryptographic Library
License:        LGPL-2.1-or-later AND GPL-2.0-or-later
Group:          Development/Libraries/C and C++
URL:            https://www.lysator.liu.se/~nisse/nettle/
Source0:        https://www.lysator.liu.se/~nisse/archive/nettle-%{version}.tar.gz
Source1:        https://www.lysator.liu.se/~nisse/archive/nettle-%{version}.tar.gz.sig
Source2:        %{name}.keyring
Source3:        baselibs.conf
# PATCH-FIX-UPSTREAM respect cflags while building
Patch0:         nettle-respect-cflags.patch
# PATCH-FIX-UPSTREAM bsc#1184401 CVE-2021-20305 Multiply function being called with out-of-range scalars
Patch1:         libnettle-CVE-2021-20305.patch
BuildRequires:  fipscheck
BuildRequires:  gmp-devel
BuildRequires:  m4
BuildRequires:  makeinfo
BuildRequires:  pkgconfig
Requires(post): %{install_info_prereq}

%description
Nettle is a cryptographic library that is designed to fit easily in more or
less any context: In crypto toolkits for object-oriented languages (C++,
Python, Pike, ...), in applications like LSH or GNUPG, or even in kernel space.

%package -n libnettle%{soname}
Summary:        Cryptographic Library
License:        LGPL-2.1-or-later
Group:          System/Libraries

%description -n libnettle%{soname}
Nettle is a cryptographic library that is designed to fit easily in more or
less any context: In crypto toolkits for object-oriented languages (C++,
Python, Pike, ...), in applications like LSH or GNUPG, or even in kernel space.

%package -n libhogweed%{hogweed_soname}
Summary:        Cryptographic Library for Public Key Algorithms
License:        LGPL-2.1-or-later
Group:          System/Libraries

%description -n libhogweed%{hogweed_soname}
Nettle is a cryptographic library that is designed to fit easily in more or
less any context: In crypto toolkits for object-oriented languages (C++,
Python, Pike, ...), in applications like LSH or GNUPG, or even in kernel space.

The libhogweed library contains public key algorithms to use with libnettle.

%package -n libnettle-devel
Summary:        Cryptographic Library
License:        LGPL-2.1-or-later
Group:          Development/Libraries/C and C++
Requires:       glibc-devel
Requires:       gmp-devel
Requires:       libhogweed%{hogweed_soname} = %{version}
Requires:       libnettle%{soname} = %{version}

%description -n libnettle-devel
Nettle is a cryptographic library that is designed to fit easily in more or
less any context: In crypto toolkits for object-oriented languages (C++,
Python, Pike, ...), in applications like LSH or GNUPG, or even in kernel space.

%package -n nettle
Summary:        Cryptographic Tools
License:        LGPL-2.1-or-later AND GPL-2.0-or-later
Group:          Productivity/Security

%description -n nettle
Nettle is a cryptographic library that is designed to fit easily in more or
less any context: In crypto toolkits for object-oriented languages (C++,
Python, Pike, ...), in applications like LSH or GNUPG, or even in kernel space.

This package contains a few command-line tools to perform cryptographic
operations using the nettle library.

%prep
%setup -q -n nettle-%{version}
%patch0 -p1
%patch1 -p1

%build
%configure \
    --disable-static \
    --enable-shared \
    --enable-fat
make %{?_smp_mflags}

%install
%make_install

# the hmac hashes:
#
# this is a hack that re-defines the __os_install_post macro
# for a simple reason: the macro strips the binaries and thereby
# invalidates a HMAC that may have been created earlier.
# solution: create the hashes _after_ the macro runs.
#
# this shows up earlier because otherwise the %%expand of
# the macro is too late.
# remark: This is the same as running
#   openssl dgst -sha256 -hmac 'orboDeJITITejsirpADONivirpUkvarP'
%{expand:%%global __os_install_post {%__os_install_post
%{_bindir}/fipshmac %{buildroot}%{_libdir}/libnettle.so.%{soname}
%{_bindir}/fipshmac %{buildroot}%{_libdir}/libhogweed.so.%{hogweed_soname}
}}

%post   -n libnettle%{soname} -p /sbin/ldconfig
%postun -n libnettle%{soname} -p /sbin/ldconfig
%post   -n libhogweed%{hogweed_soname} -p /sbin/ldconfig
%postun -n libhogweed%{hogweed_soname} -p /sbin/ldconfig
%post -n libnettle-devel
%install_info --info-dir="%{_infodir}" "%{_infodir}"/nettle.info%{ext_info}

%preun -n libnettle-devel
%install_info_delete --info-dir="%{_infodir}" "%{_infodir}"/nettle.info%{ext_info}

%check
make check %{?_smp_mflags}

%files -n libnettle%{soname}
%license COPYING*
%doc AUTHORS ChangeLog NEWS README TODO
%{_libdir}/libnettle.so.%{soname}
%{_libdir}/libnettle.so.%{soname}.*
%{_libdir}/.libnettle.so.%{soname}.hmac

%files -n libhogweed%{hogweed_soname}
%{_libdir}/libhogweed.so.%{hogweed_soname}
%{_libdir}/libhogweed.so.%{hogweed_soname}.*
%{_libdir}/.libhogweed.so.%{hogweed_soname}.hmac

%files -n libnettle-devel
%{_includedir}/nettle
%{_libdir}/libnettle.so
%{_libdir}/libhogweed.so
%{_infodir}/nettle.info%{?ext_info}
%{_libdir}/pkgconfig/hogweed.pc
%{_libdir}/pkgconfig/nettle.pc

%files -n nettle
%license COPYING*
%doc AUTHORS ChangeLog NEWS README TODO
%{_bindir}/nettle-lfib-stream
%{_bindir}/nettle-pbkdf2
%{_bindir}/pkcs1-conv
%{_bindir}/sexp-conv
%{_bindir}/nettle-hash

%changelog
* Mon Apr 19 2021 Pedro Monreal <pmonreal@suse.com>
- Security fix: [bsc#1184401, CVE-2021-20305]
  * multiply function being called with out-of-range scalars
  * Affects ecc-ecdsa-sign(), ecc_ecdsa_verify() and _eddsa_hash().
- Add libnettle-CVE-2021-20305.patch
* Tue Oct  1 2019 Vítězslav Čížek <vcizek@suse.com>
- Install checksums for binary integrity verification which are
  required when running in FIPS mode (bsc#1152692, jsc#SLE-9518)
* Sun Mar 17 2019 Jason Sikes <jsikes@suse.de>
- Update to 3.4.1 - FATE#327114 (bsc#1129598)
  * Fix CVE-2018-16869 (bsc#1118086)
    libnettle-CVE-2018-16869-3.4.patch (removed)
    All functions using RSA private keys are now side-channel
    silent, meaning that they try hard to avoid any branches or
    memory accesses depending on secret data. This applies both to
    the bignum calculations, which now use GMP's mpn_sec_* family
    of functions, and the processing of PKCS#1 padding needed for
    RSA decryption.
  * Changes in behavior:
    The functions rsa_decrypt and rsa_decrypt_tr may now clobber
    all of the provided message buffer, independent of the
    actual message length. They are side-channel silent, in that
    branches and memory accesses don't depend on the validity or
    length of the message. Side-channel leakage from the
    caller's use of length and return value may still provide an
    oracle useable for a Bleichenbacher-style chosen ciphertext
    attack. Which is why the new function rsa_sec_decrypt is
    recommended.
  * New features:
    A new function rsa_sec_decrypt.
  * Bug fixes:
  - Fix bug in pkcs1-conv, missing break statements in the
    parsing of PEM input files.
  - Fix link error on the pss-mgf1-test test, affecting builds
    without public key support.
* Fri Dec 14 2018 Pedro Monreal Gonzalez <pmonrealgonzalez@suse.com>
- Security fix: [bsc#1118086, CVE-2018-16869]
  * Leaky data conversion exposing a manager oracle
  * Added libnettle-CVE-2018-16869-3.4.patch
* Thu Feb 22 2018 fvogt@suse.com
- Use %%license (boo#1082318)
* Sun Nov 19 2017 astieger@suse.com
- libnettle 3.4:
  * Fixed an improper use of GMP mpn_mul, breaking curve2559 and
    eddsa on certain platforms
  * Fixed memory leak when handling invalid signatures in
    ecdsa_verify. Fix contributed by Nikos Mavrogiannopoulos.
  * Reorganized the way certain data items are made available:
    Nettle header files now define the symbols
    nettle_hashes, nettle_ciphers, and nettle_aeads, as
    preprocessor macros invoking a corresponding accessor
    function. For backwards ABI compatibility, the symbols are
    still present in the compiled libraries, and with the same
    sizes as in nettle-3.3.
  * Support for RSA-PSS signatures
  * Support for the HKDF key derivation function, defined by RFC
    5869
  * Support for the Cipher Feedback Mode (CFB)
  * New accessor functions: nettle_get_hashes,
    nettle_get_ciphers, nettle_get_aeads, nettle_get_secp_192r1,
    nettle_get_secp_224r1, nettle_get_secp_256r1,
    nettle_get_secp_384r1, nettle_get_secp_521r1.
    Direct access to data items is deprecated going forward.
  * The base16 and base64 functions now use the type char * for
    ascii data, rather than uint8_t *. This eliminates the last
    pointer-signedness warnings when building Nettle
  * The contents of the header file nettle/version.h is now
    architecture independent, except in --enable-mini-gmp
  * Prevent data sizes from leaking into the ABI
- Fixes previously carried as patches:
  * Fix compilation error with --enable-fat om ARM
    Drop nettle-3.3-fix-fat-arm.patch
* Mon Sep  4 2017 asn@cryptomilk.org
- Add patch to fix build of fat-arm:
  * nettle-3.3-fix-fat-arm.patch
* Sun Sep  3 2017 asn@cryptomilk.org
- Build nettle with AES-NI support (bsc#1056980)
* Thu Feb  9 2017 dimstar@opensuse.org
- Explicitly BuildRequire m4
* Fri Oct 28 2016 astieger@suse.com
- libnettle 3.3:
  * Invalid private RSA keys, with an even modulo, are now
    rejected by rsa_private_key_prepare. (Earlier versions
    allowed such keys, even if results of using them were bogus).
    Nettle applications are required to call
    rsa_private_key_prepare and check the return value, before
    using any other RSA private key functions; failing to do so
    may result in crashes for invalid private keys.
  * Ignore bit 255 of the x coordinate of the input point to
    curve25519_mul, as required by RFC 7748. To differentiate at
    compile time, curve25519.h defines the constant
    NETTLE_CURVE25519_RFC7748.
  * RSA and DSA now use side-channel silent modular
    exponentiation, to defend against attacks on the private key
    from evil processes sharing the same processor cache. This
    attack scenario is of particular relevance when running an
    HTTPS server on a virtual machine, where you don't know who
    you share the cache hardware with.
    bsc#991464 CVE-2016-6489
  * Fix sexp-conv crashes on invalid input
  * Fix out-of-bounds read in des_weak_p
  * Fix a couple of formally undefined shift operations
  * Fix compilation with c89
  * New function memeql_sec, for side-channel silent comparison
    of two memory areas.
  * Building the public key support of nettle now requires GMP
    version 5.0 or later (unless --enable-mini-gmp is used).
* Tue Feb 23 2016 tchvatal@suse.com
- Fix postun->preun on info packages regenerating
* Thu Jan 28 2016 tchvatal@suse.com
- Version update to 3.2 release bnc#964849 CVE-2015-8805 bnc#964847
  CVE-2015-8804 bnc#964845 CVE-2015-8803:
  * New functions for RSA private key operations, identified by
    the "_tr" suffix, with better resistance to side channel
    attacks and to hardware or software failures which could
    break the CRT optimization
  * SHA3 implementation is updated according to the FIPS 202 standard
  * New ARM Neon implementation of the chacha stream cipher
  * Should be compatible binary with 3.1 series
- Add patch to fix build with cflags:
  * nettle-respect-cflags.patch
* Mon Jun 22 2015 tchvatal@suse.com
- Remove off-by-one-test-suite.patch as it was fixed by upstream
  differently
* Sun Apr 26 2015 astieger@suse.com
- nettle 3.1.1
  Non-critical bugfix release, binary compatible to 3.1
  * By accident, nettle-3.1 disabled the assembly code for the
    secp_224r1 and secp_521r1 elliptic curves on all x86_64
    configurations, making signature operations on those curves
    10%%-30%% slower. This code is now re-enabled.
  * The x86_64 assembly implementation of gcm hashing has been
    fixed to work with the Sun/Oracle assembler.
* Thu Apr 23 2015 vpereira@suse.com
  added patch: off-by-one-test-suite.patch
- Address Sanitizer, found a off-by-one error in the test suite (bnc#928328)
* Sat Apr 11 2015 astieger@suse.com
- nettle 3.1 (libnettle6, libhogweed4)
- bug fixes in 3.1:
  * Fixed a missing include of <limits.h>, which made the camellia
    implementation fail on all 64-bit non-x86 platforms.
  * Eliminate out-of-bounds reads in the C implementation of memxor
  (related to valgrind's --partial-loads-ok flag). [bso#926745)
- interface changes in 3.1:
  * Declarations of many internal functions are moved from ecc.h to
    ecc-internal.h.
- interface changes in 3.0:
  * contains developer relevant incompatible interface changes
- Removed features:
  * nettle_next_prime, use GMP's mpz_nextprime
  * Deleted the RSAREF compatibility
- New features in 3.1:
  * Support for curve25519 and for EdDSA25519 signatures.
  * Support for "fat builds" on x86_64 and arm (not enabled)
  * Support for building the hogweed library (public key support)
    using "mini-gmp" (not enabled)
  * The shared libraries are now built with versioned symbols.
  * Support for "URL-safe" base64 encoding and decoding
- New features in 3.0:
  * new DSA, AES, Camellia interfaces
  * Support for Poly1305-AES MAC.
  * Support for the ChaCha stream cipher and EXPERIMENTAL
    support for the ChaCha-Poly1305 AEAD mode.
  * Support for EAX mode.
  * Support for CCM mode.
  * Additional variants of SHA512 with output size of 224 and 256 bits
  * New interface, struct nettle_aead, for mechanisms providing
    authenticated encryption with associated data (AEAD).
  * DSA: Support a wider range for the size of q and a wider
    range for the digest size.
  * New command line tool nettle-pbkdf2.
- Optimizations in 3.1:
  * New x86_64 implementation of AES, using the "aesni" instructions
- Optimizations in 3.0:
  * New x86_64 assembly for GCM and MD5. Modest speedups on the
    order of 10%%-20%%.
* Fri Mar 13 2015 tchvatal@suse.com
- Add url to the spec
* Thu Mar  5 2015 mpluskal@suse.com
- Revert back to 2.7
* Tue May 13 2014 tchvatal@suse.com
- Cleanup with spec-cleaner
- Paralelize test run
* Thu Dec 19 2013 meissner@suse.com
- also build baselibs for libnettle-devel (for wine 32bit development)
* Tue Jun 25 2013 meissner@suse.com
- Update to version 2.7.1
  * Fixed ecc_modp_mul call, to avoid invalid overlap of arguments to
    mpn_mul_n. Problem tracked down by Magnus Holmgren.
  * ARM fixes.
- reference gpg signatures and keyring. checking not enabled as to
  avoid cycles.
* Thu May 16 2013 idonmez@suse.com
- Update to version 2.7
  * Support for the GOST R 34.11-94 hash algorithm
  * Support for SHA3
  * Support for PKCS #5 PBKDF2
  * Fixed a small memory leak in nettle_realloc and
    nettle_xrealloc.
  * x86_64 assembly for SHA256, SHA512, and SHA3
  * ARM assembly code for several additional algorithms,
    including AES, Salsa20, and the SHA family of hash
    functions.
  * Support for 12-round salsa20, "salsa20r12", as specified by
    eSTREAM.
  * Support for UMAC, including x86_64 and ARM assembly.
  * Support for ECDSA signatures. Elliptic curve operations over
    the following curves: secp192r1, secp224r1, secp256r1,
    secp384r1 and secp521r1, including x86_64 and ARM assembly
    for the most important primitives.
- Depend on makeinfo for info file generation.
- Don't disable static libs, they are needed at build time.
* Sun Nov 25 2012 andreas.stieger@gmx.de
- upgrade to 2.5:
  * removed some internal undocumented functions
  * pkcs1_signature_prefix renamed to _pkcs1_signature_prefix
    with slightly different behavior
  * nettle-internal.c is no longer included
  * Support for the salsa20 block cipher
  * Tentative interface for timing-resistant RSA functions
  * A more general interface for PKCS#1 signatures
  * Documentation, example programs for the base16 and base64
  * Use an additional table to avoid GF2^8 multiplications in
    aes_invert_key (mainly used by aes_set_decrypt_key). Also
    tabulate round constants in aes_set_encrypt_key.
- configure --enable-shared now default, no longer required
* Tue Jan 31 2012 jengelh@medozas.de
- Remove redundant tags/sections per specfile guideline suggestions
* Sat Sep 24 2011 crrodriguez@opensuse.org
- BuildRequire pkgconfig to fix rpmlint warning
* Mon Sep 19 2011 crrodriguez@opensuse.org
- Update to version 2.4 only two important changes
  * ripemd160 was broken on all big endian machines
  * add pkgconfig files
* Mon Aug 29 2011 coolo@novell.com
- use original source
- add baselibs.conf
- drop licenses
* Wed Aug 17 2011 crrodriguez@opensuse.org
- Fix licenses [bnc#712616]
- run make check
* Tue Aug 16 2011 crrodriguez@opensuse.org
- Update to version 2.2, bump sonames accordingly
- Fix build in factory
- Fix -devel package dependencies
- Tune up spec file in order to submit package to factory
  as is needed for gnutls version 3.x
* Tue Jan  5 2010 pascal.bleser@opensuse.org
- initial package (2.0)
